@@ -8,13 +8,9 @@ import numpy as np
 
 from kmer_mapper.mapper import map_kmers_to_graph_index
 from kmer_mapper.util import remap_array
-from scipy.ndimage import convolve1d
-import pandas as pd
-import scipy.signal
 from graph_kmer_index import KmerIndex
-from shared_memory_wrapper import from_shared_memory, to_shared_memory, SingleSharedArray
+from shared_memory_wrapper import from_shared_memory, to_shared_memory, SingleSharedArray, object_to_shared_memory
 from shared_memory_wrapper.shared_memory import get_shared_pool, close_shared_pool, object_from_shared_memory
-from pathos.multiprocessing import Pool
 from itertools import repeat
 from graph_kmer_index.collision_free_kmer_index import MinimalKmerIndex
 
@@ -23,6 +19,8 @@ from .parser import OneLineFastaParser, Sequences, OneLineFastaParser2bit, Buffe
 from .util import log_memory_usage_now
 from .kmers import KmerHash, TwoBitHash
 from npstructures import Counter
+from bionumpy.kmers import fast_hash
+import bionumpy as bnp
 
 
 def map_kmers_to_graph_index_wrapper(*args):
@@ -128,14 +126,12 @@ def map_fasta_single_thread_with_numpy_parsing(data):
     logging.info("Reeading chunk of reads from shared memory")
     raw_chunk = from_shared_memory(args.buffer_type, reads)
     sequence_chunk = raw_chunk.get_sequences()
-    logging.info("Size of sequence chunk (GB): %.3f" % (sequence_chunk.nbytes() / 1000000000))
 
     if not args.use_numpy:
         node_counts = from_shared_memory(SingleSharedArray, "counts_shared"+args.random_id).array
 
     t = time.perf_counter()
     hashes = TwoBitHash(k=args.kmer_size).get_kmer_hashes(sequence_chunk)
-    logging.info("Size of hashes (GB): %.3f" % (hashes.nbytes / 1000000000))
     logging.info("Time spent to get %d kmer hashes: %.3f" % (len(hashes), time.perf_counter() - t))
 
     t = time.perf_counter()
@@ -148,6 +144,7 @@ def map_fasta_single_thread_with_numpy_parsing(data):
         logging.info("Using cython counter")
         kmer_index = from_shared_memory(KmerIndex, args.kmer_index)
         node_counts += map_kmers_to_graph_index(kmer_index, args.max_node_id, hashes, args.max_hits_per_kmer)
+
     logging.info("Getting node counts took %.3f sec" % (time.perf_counter()-t))
     logging.info("Done with chunk. Took %.3f sec" % (time.perf_counter()-time_start))
 
@@ -173,7 +170,7 @@ def map_fasta(args, kmer_index):
         node_counts = None
 
 
-    parser = BufferedNumpyParser.from_filename(args.fasta_file, args.chunk_size * 130)
+    parser = BufferedNumpyParser.from_filename(args.fasta_file, args.chunk_size)
     args.buffer_type = OneLineFastaBuffer2Bit
     chunks = parser.get_chunks()
     reads = (to_shared_memory(chunk) for chunk in chunks)
@@ -206,3 +203,88 @@ def map_fasta(args, kmer_index):
     logging.info("Spent %.3f sec in total mapping kmers using %d threads" % (time.time()-start_time, args.n_threads))
 
     close_shared_pool()
+    
+from bionumpy.kmers import fast_hash
+from bionumpy.kmers import KmerEncoding
+
+
+def _parallel_sequence_map_wrapper(data):
+    index_id, max_node_id, k, sequence_id = data
+    sequence = object_from_shared_memory(sequence_id)
+    sequence = sequence.ravel()  # .astype(np.int64)
+    #hasher = KmerEncoding(k, None, 4)
+    #hashes = hasher.rolling_window(sequence)
+    #kmers = hashes.astype(np.uint64)
+    t0 = time.perf_counter()
+    kmers = fast_hash(sequence[::-1], k).astype(np.uint64)
+    logging.info("Hashing %d kmers took %.4f sec" % (len(kmers), time.perf_counter()-t0))
+    index = object_from_shared_memory(index_id)
+
+
+    t0 = time.perf_counter()
+    mapped = map_kmers_to_graph_index(index, max_node_id, kmers)
+    logging.info("Mapping %d kmers took %.4f sec" % (len(kmers), time.perf_counter()-t0))
+    return mapped
+
+
+def _parallel_map_wrapper(data):
+    index_id, max_node_id, kmer_id = data
+    kmers = object_from_shared_memory(kmer_id)
+    index = object_from_shared_memory(index_id)
+    return map_kmers_to_graph_index(index, max_node_id, kmers)
+
+
+class ParalellMapper:
+    def __init__(self, kmer_index, max_node_id, n_threads=8):
+        t = time.perf_counter()
+        self._max_node_id = max_node_id
+        self._kmer_mapper = kmer_index
+        self._kmer_mapper = object_to_shared_memory(kmer_index)
+        self._n_threads = n_threads
+        self._counts = np.zeros(max_node_id, dtype=np.uint32)
+        self._pool = get_shared_pool(self._n_threads)
+        logging.info("Time spent initing ParallellMapper: %.4f" % (time.perf_counter()-t))
+
+    def map_reads_file(self, read_file_name, k, chunk_size=10000000):
+        chunks = bnp.open(read_file_name, chunk_size=chunk_size)
+        #chunks = (chunk.sequence.ravel() for chunk in chunks)
+        sequences = (object_to_shared_memory(chunk.sequence) for chunk in chunks)
+
+        for result in self._pool.imap(_parallel_sequence_map_wrapper,
+                                  zip(itertools.repeat(self._kmer_mapper), itertools.repeat(self._max_node_id - 1),
+                                      itertools.repeat(k),
+                                      sequences)):
+            self._counts += result
+
+    def map_numeric_sequence(self, numeric_sequence, k):
+        t = time.perf_counter()
+        assert numeric_sequence.dtyp == np.uint8
+        sequences = np.array_split(numeric_sequence, self._n_threads)
+        logging.info("time spent splitting sequences: %.3f" % (time.perf_counter()-t))
+
+        t = time.perf_counter()
+        sequences = [object_to_shared_memory(k) for k in sequences]
+        func = self._pool.imap
+        if self._n_threads == 1:
+            func = map
+
+        logging.info("time spent writing sequences to shared memory: %.3f" % (time.perf_counter()-t))
+        for result in func(_parallel_sequence_map_wrapper,
+                                      zip(itertools.repeat(self._kmer_mapper), itertools.repeat(self._max_node_id - 1), itertools.repeat(k),
+                                          sequences)):
+            self._counts += result
+
+    def map(self, kmers):
+        assert kmers.dtype == np.uint64
+        kmers = np.array_split(kmers, self._n_threads)
+        kmers = (object_to_shared_memory(k) for k in kmers)
+        for result in self._pool.imap(_parallel_map_wrapper,
+                                      zip(itertools.repeat(self._kmer_mapper), itertools.repeat(self._max_node_id-1), kmers)):
+            self._counts += result
+
+    def get_results(self):
+        #logging.info("Sum of counts: %d" % np.sum(self._counts))
+        return self._counts
+
+    def reset(self):
+        self._counts[:] = 0
\ No newline at end of file
